/* ========================================================================== */
/*                                                                            */
/*   Filename.c                                                               */
/*   (c) 2001 Author  E.Supler                                                          */
/*                                                                            */
/*   Description  AVR                                                              */
/*                                                                            */
/* ========================================================================== */
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include "globals.h"




#define datapin     0 	// define datapin from atmega8 to 74HC164
#define clockpin    1 	// define clock pin from atmega8 to 74HC164
#define Epin		2	// controllerpin that enables LCD

/*
LCD pins connected to 74hc164:
Q7 ->LCD_D0
Q6 ->LCD_D1
Q5 ->LCD_D2
Q4 ->LCD_D3
Q3 ->LCD_D4
Q2 ->LCD_D5
Q1 ->LCD_D6
Q0 ->LCD_D7
*/


void sendByteToRegister(uint8_t LCDdata)
{
  uint8_t i, temp;
  PORTC&=~_BV(datapin);			// sets datapin to output a LOW
  for (i=0;i<=7;i++) 				//clear shift 74HC164 register
	{
		PORTC|=_BV(clockpin);		// sets clockpin to output a HIGH
		_delay_ms(0.5);
		PORTC&= ~_BV(clockpin);	// sets clockpin to output a LOW
		_delay_ms(0.5);
	}
  temp=LCDdata;	
  for (i=0;i<=7;i++)		    	//write 8bit LCDdata to 74HC164 register
	{
		PORTC |= (LCDdata&1);
		_delay_ms(0.5);
		PORTC |= _BV(clockpin);	// sets clockpin to output a HIGH
		_delay_ms(0.5);
		PORTC&= ~_BV(clockpin);	// sets clockpin to output a LOW
		_delay_ms(0.5);
		PORTC&=~_BV(datapin);	// sets datapin to output a LOW
		_delay_ms(0.5);
		LCDdata=temp>>1;
		temp=LCDdata;	
	}
}

void enableCommand()
{
  PORTC |= _BV(Epin);
}
void disableCommand()
{
  PORTC &= ~_BV(Epin);
}
void enableData()
{
  PORTC |= _BV(datapin);
  PORTC |= _BV(Epin);
}
void disableData()
{
  PORTC &= ~_BV(Epin);
  PORTC &= ~_BV(datapin);
}

void lcd_send_char(uint8_t letter) //forms data ready to send to 74HC164
{
  sendByteToRegister(letter);//sends char to shift register
  enableData();
  _delay_ms(0.5);
  disableData();
}
void sendCommand(uint8_t cmd) //forms data ready to send to 74HC164
{
    sendByteToRegister(cmd);//sends command to shift register
    enableCommand();
    _delay_ms(0.5);
    disableCommand();
}
void lcd_init(void)
{
// init Atmega8
  PORTC&=~((1<<datapin)|(1<<clockpin)|(1<<Epin));//set outputs to zerro
  DDRC|=(1<<datapin)|(1<<clockpin)|(1<<Epin); //enable output pins
// lcd init
  _delay_ms(30); //wait for poverup
  sendByteToRegister(0x30);//1
  enableCommand();
  _delay_ms(10);
  disableCommand();
  sendByteToRegister(0x30);//2
  enableCommand();
  _delay_ms(10);
  disableCommand();
  sendByteToRegister(0x30);//3
  enableCommand();
  _delay_ms(20);
  disableCommand();
  _delay_ms(10);//wait for more
  sendByteToRegister(0x38);//enable 8 bit mode dual line
  enableCommand();
  _delay_ms(0.5);
  disableCommand();
  sendByteToRegister(0x0F);// increment adress counter. Cursor shift

  enableCommand();
  _delay_ms(0.5);
  disableCommand();
}

void lcd_gotoxy(uint8_t x,uint8_t y)
{
 if(x<40)
 {
    if(y) x|=0b01000000;
    x|=0b10000000;
    sendCommand(x);
  }
}

void lcd_puts(const char *msg)
{
	/*****************************************************************
	
	This function Writes a given string to lcd at the current cursor
	location.

	Arguments:
	msg: a null terminated string to print


	*****************************************************************/
 while(*msg!='\0')
 {
	lcd_send_char(*msg);
	msg++;
 }
}
void lcd_puts_p(const uint8_t *FlashLoc)
{
    uint8_t i;
    for(i=0;(uint8_t)pgm_read_byte(&FlashLoc[i]);i++)
	  {
		 lcd_send_char((uint8_t)pgm_read_byte(&FlashLoc[i]));
	  }

}


void lcd_putint(int8_t val,int8_t field_length)
{
	/***************************************************************
	This function writes a integer type value to LCD module

	Arguments:
	1)int val	: Value to print

	2)unsigned int field_length :total length of field in which the value is printed
	must be between 1-5 if it is -1 the field length is no of digits in the val

	****************************************************************/
//	if(val<0) lcd_send_char('-');
    char str[5]={0,0,0,0,0};
    int8_t val1;
	int8_t i=4,j=0;
	val1 = val;
    if (val < 0) val = abs(val);

	while(val)
	{
	str[i]=val%10;
	val=val/10;
	i--;
	}
	if(field_length==-1)
		while(str[j]==0) j++;
	else
		j=5-field_length;

	if(val1<0)
	  {
	     lcd_send_char('-');
		 j++;
      }
	for(i=j;i<5;i++)
	{
	lcd_send_char(48+str[i]);
	}
	
}

